/*
    UHU-menu -- output plugin for KDE (3.1.4)
    Copyright (C) 2003  Koblinger Egmont <egmont@uhulinux.hu>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#define _GNU_SOURCE
#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <limits.h>
#include <errno.h>

#include "uhu-menu.h"

static void write_item (FILE *f, const struct item *item)
{
	int first = 1;
	struct section *s = item->sections;
	struct key *k;
	struct lang *l;
	char *p;
	while (s != NULL) {
		if (!first) fprintf(f, "\n");
		first = 0;
		fprintf(f, "[%s]\n", s->name);
		k = s->keys;
		while (k != NULL) {
			l = k->langs;
			while (l != NULL) {
				if (l->language->name[0] != '\0') {
					fprintf(f, "%s[%s]=", k->name, l->language->name);
				} else {
					fprintf(f, "%s=", k->name);
				}
				p = l->value;
				while (*p != '\0') {
					switch (*p) {
					  case ' ':
						if (p == l->value || *(p+1) == '\0') fprintf(f, "\\s");
						else fputc(' ', f);
						break;
					  case '\n':
						fprintf(f, "\\n");
						break;
					  case '\t':
						fprintf(f, "\\t");
						break;
					  case '\r':
						fprintf(f, "\\r");
						break;
					  case '\\':
						fprintf(f, "\\\\");
						break;
					  default:
						fputc(*p, f);
					}
					p++;
				}
				fputc('\n', f);
				l = l->next;
			}
			k = k->next;
		}
		s = s->next;
	}
}

static void dump (const struct item *item, const char *path)
{
	int i, l;
	FILE *f;
	char buf[PATH_MAX];
	int settingsmenu = 0;

	if (!strcmp(item->file_name, "/Settings") ||
	    !strncmp(item->file_name, "/Settings/", 10)) settingsmenu = 1;

	if (settingsmenu) {
		sprintf(buf, "%s/Settingsmenu%s", path, item->file_name + 9);
	} else {
		sprintf(buf, "%s%s", path, item->file_name);
	}
	if (!item->issubmenu) {
		l = strlen(buf);
		if (l < 8 || strcmp(buf + l - 8, ".desktop")) strcpy(buf + l, ".desktop");
	} else {
		mkdir(buf, 0777);
		if (settingsmenu) {
			sprintf(buf, "%s/Settingsmenu%s/.directory", path, item->file_name + 9);
		} else {
			sprintf(buf, "%s%s/.directory", path, item->file_name);
		}
	}
	f = fopen(buf, "w");
	if (f == NULL) {
		message_e(3, "fopen %s", buf);
	} else {
		write_item(f, item);
		fclose(f);
	}
	if (item->issubmenu) {
		for (i = 0; i < item->subitems_nr; i++) {
			dump(&item->subitems[i], path);
		}
	}
}

int write_menu (const struct item *item, const char *path)
{
	char path_tmp[PATH_MAX];
	char buf[PATH_MAX];

	sprintf(path_tmp, "%s.tmp", path);
	rm_rf(path_tmp);
	mkdir(path_tmp, 0777);
	strcpy(buf, path_tmp);
	strcat(buf, "/share");
	mkdir(buf, 0777);
	strcat(buf, "/applnk");
	mkdir(buf, 0777);
	dump(item, buf);
	movetree(path_tmp, path);
	return 0;
}

